--[[-----------------------------------------------------------------------------
ItemInfo Widget
Displays an item icon with name, Reforging, and equipment slot info
-------------------------------------------------------------------------------]]
local Type, Version = "PlayerUpgradeViewLine", 34
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, select, pairs = math.max, select, pairs

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

-- Global vars/functions that we don't upvalue since they might get hooked, or upgraded
-- List them here for Mikk's FindGlobals script
-- GLOBALS: GameFontHighlightSmall

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]

local function Layout(self)
	local frame = self.frame
	local name = self.name
	
	if not self.resizing then
		self.resizing = true
		name:SetHeight(0)
		name:SetWidth(self.name:GetWidth())
		frame:SetHeight(name:GetHeight() + 14)
		self.resizing = false
	end

end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		self.resizing = true
		self.name:SetHeight(0)
		self.name:SetWidth(270)
		self.frame:SetHeight(self.name:GetHeight() + 14)
		self.resizing = false
		Layout(self)
	end,

	["OnWidthSet"] = function(self, width)
		Layout(self)
	end,

	["SetPlayerText"] = function(self, player, class)
		self.player:SetText(player)
		self.player:SetTextColor(RAID_CLASS_COLORS[class].r, RAID_CLASS_COLORS[class].g, RAID_CLASS_COLORS[class].b)
	end,

	["SetNameText"] = function(self, name, quality)
		self.name:SetText(name)
		self.name:SetTextColor(GetItemQualityColor(quality))
	end,

	["SetEquippedItemScoreText"] = function(self, text)
		self.equippedItemScore:SetText(text)
	end,

	["SetDroppedItemScoreText"] = function(self, text)
		self.droppedItemScore:SetText(text)
	end,

	["SetPctUpgradeText"] = function(self, text)
		if text then
			self.pctUpgrade:SetText(text .. "%")
			if (tonumber(text)) > 0 then
				self.pctUpgrade:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b)
			else
				self.pctUpgrade:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b)
			end
		else
			self.pctUpgrade:SetText("")
		end
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local PaneBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 5, right = 3, top = 3, bottom = 5 }
}

local function Constructor()
	local playerWidth = 124
	local nameWidth = 195
	local scoreWidth = 62
	local upgradeWidth = 69

	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetFrameStrata("FULLSCREEN_DIALOG")
	frame:SetWidth(526)
	frame:SetBackdrop(PaneBackdrop)
	frame:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	frame:SetBackdropBorderColor(0.4, 0.4, 0.4)

	local player = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	player:SetJustifyH("LEFT")
	player:SetFont("Fonts\\FRIZQT__.TTF", 14)
	player:SetText("Player")
	player:SetPoint("TOPLEFT", frame, "TOPLEFT", 7, -7)
	player:SetPoint("RIGHT", frame, "LEFT", playerWidth + 7, 0)

	local name = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	name:SetJustifyV("TOP")
	name:SetJustifyH("LEFT")
	name:SetFont("Fonts\\FRIZQT__.TTF", 14)
	name:SetWordWrap(true)
	name:SetText("Name")
	name:SetHeight(0)
	name:SetPoint("TOPLEFT", player, "TOPRIGHT")

	local pctUpgrade = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	pctUpgrade:SetJustifyH("RIGHT")
	pctUpgrade:SetFont("Fonts\\FRIZQT__.TTF", 14)
	pctUpgrade:SetText("Upgrade")
	pctUpgrade:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -7, -7)
	pctUpgrade:SetPoint("LEFT", frame, "RIGHT", 0 - upgradeWidth - 7, 0)
	
	local droppedItemScore = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	droppedItemScore:SetJustifyH("RIGHT")
	droppedItemScore:SetFont("Fonts\\FRIZQT__.TTF", 14)
	droppedItemScore:SetText("Score")
	droppedItemScore:SetPoint("TOPRIGHT", pctUpgrade, "TOPLEFT")
	droppedItemScore:SetPoint("LEFT", pctUpgrade, "LEFT", 0 - scoreWidth, 0)

	local equippedItemScore = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	equippedItemScore:SetJustifyH("RIGHT")
	equippedItemScore:SetFont("Fonts\\FRIZQT__.TTF", 14)
	equippedItemScore:SetText("Score")
	equippedItemScore:SetPoint("TOPRIGHT", droppedItemScore, "TOPLEFT")
	equippedItemScore:SetPoint("LEFT", droppedItemScore, "LEFT", 0 - scoreWidth, 0)
	name:SetPoint("RIGHT", equippedItemScore, "LEFT")
	
	frame:SetHeight(name:GetStringHeight() + 14)

	local widget = {
		type = Type,
		frame = frame,
		player = player,
		name = name,
		equippedItemScore = equippedItemScore,
		droppedItemScore = droppedItemScore,
		pctUpgrade = pctUpgrade
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
